@@ -1,8 +1,6 @@
 
 /*
  * mh.h -- main header file for all of nmh
- *
- * $Id: mh.h,v 1.8 2007/11/04 11:54:32 jjr Exp $
  */
 
 #include <h/nmh.h>
@@ -29,6 +27,15 @@
 #endif
 typedef unsigned char  boolean;  /* not int so we can pack in a structure */
 
+/* If we're using gcc then give it some information about
+ * functions that abort.
+ */
+#if __GNUC__ > 2
+#define NORETURN __attribute__((__noreturn__))
+#else
+#define NORETURN
+#endif
+
 /*
  * user context/profile structure
  */
@@ -214,7 +221,15 @@ struct msgs {
  * m_getfld() message parsing
  */
 
-#define NAMESZ  128		/* Limit on component name size     */
+#define NAMESZ  999		/* Limit on component name size.
+				   RFC 2822 limits line lengths to
+				   998 characters, so a header name
+				   can be at most that long.
+				   m_getfld limits header names to 2
+				   less than NAMESZ, which is fine,
+				   because header names must be
+				   followed by a colon.	 Add one for
+				   terminating NULL. */
 
 #define LENERR  (-2)		/* Name too long error from getfld  */
 #define FMTERR  (-3)		/* Message Format error             */
@@ -336,7 +351,7 @@ extern char *vmhproc;
 extern char *whatnowproc;
 extern char *whomproc;
 
-extern void (*done) (int);
+extern void (*done) (int) NORETURN;
 
 #include <h/prototypes.h>
 
