@@ -2,8 +2,6 @@
 /*
  * hosts.c -- find out the official name of a host
  *
- * $Id: hosts.c,v 1.8 2007/04/13 11:53:08 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -18,6 +16,7 @@
 
 #include <h/mh.h>
 #include <h/mts.h>
+#include <sys/socket.h>
 #include <netdb.h>
 
 static struct host {
@@ -38,7 +37,7 @@ OfficialName (char *name)
 {
     unsigned char *p;
     char *q, site[BUFSIZ];
-    struct hostent *hp;
+    struct addrinfo hints, *res;
 
     static char buffer[BUFSIZ];
     char **r;
@@ -52,12 +51,14 @@ OfficialName (char *name)
     if (!mh_strcasecmp (LocalName(), site))
 	return LocalName();
 
-#ifdef	HAVE_SETHOSTENT
-    sethostent (1);
-#endif
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_flags = AI_CANONNAME;
+    hints.ai_family = PF_UNSPEC;
 
-    if ((hp = gethostbyname (q))) {
-	strncpy (buffer, hp->h_name, sizeof(buffer));
+    if (getaddrinfo(q, NULL, &hints, &res) == 0) {
+	strncpy (buffer, res->ai_canonname, sizeof(buffer));
+	buffer[sizeof(buffer) - 1] = '\0';
+	freeaddrinfo(res);
 	return buffer;
     }
     if (hosts.h_name || init_hs ()) {
