@@ -2,8 +2,6 @@
 /*
  * addrsbr.c -- parse addresses 822-style
  *
- * $Id: addrsbr.c,v 1.7 2006/03/08 12:14:16 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -119,7 +117,7 @@ getm (char *str, char *dfhost, int dftype, int wanthost, char *eresult)
     char *dp;
 #endif /* not DUMB */
 
-    if (err && err[0]) {
+    if (err[0]) {
 	if (eresult)
 	    strcpy (eresult, err);
 	else
