@@ -2,8 +2,6 @@
 /*
  * fmt_scan.c -- format string interpretation
  *
- * $Id: fmt_scan.c,v 1.25 2008/04/05 18:41:37 pm215 Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -297,7 +295,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 {
     char *cp, *ep;
     unsigned char *sp;
-    char *savestr;
+    char *savestr = NULL;
     unsigned char *str = NULL;
     char buffer[BUFSIZ], buffer2[BUFSIZ];
     int i, c, ljust, n;
