@@ -2,8 +2,6 @@
 /*
  * getans.c -- get an answer from the user and return a string array
  *
- * $Id: getans.c,v 1.3 2007/04/29 22:34:04 pm215 Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -50,7 +48,7 @@ getans (char *prompt, struct swit *ansp)
 	*cp = '\0';
 	if (ansbuf[0] == '?' || cp == ansbuf) {
 	    printf ("Options are:\n");
-	    print_sw (ALL, ansp, "");
+	    print_sw (ALL, ansp, "", stdout);
 	    continue;
 	}
 	cpp = brkstring (ansbuf, " ", NULL);
