@@ -2,8 +2,6 @@
 /*
  * pidwait.c -- wait for child to exit
  *
- * $Id: pidwait.c,v 1.6 2008/04/11 14:12:55 opk Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -22,7 +20,7 @@ int
 pidwait (pid_t id, int sigsok)
 {
     pid_t pid;
-    SIGNAL_HANDLER istat, qstat;
+    SIGNAL_HANDLER istat = NULL, qstat = NULL;
 
 #ifdef HAVE_UNION_WAIT
     union wait status;
