@@ -2,8 +2,6 @@
 /*
  * inc.c -- incorporate messages from a maildrop into a folder
  *
- * $Id: inc.c,v 1.26 2007/11/04 11:54:34 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -37,10 +35,6 @@
 # include <h/popsbr.h>
 #endif
 
-#ifdef HESIOD
-# include <hesiod.h>
-#endif
-
 #include <h/fmt_scan.h>
 #include <h/scansbr.h>
 #include <h/signals.h>
@@ -55,24 +49,6 @@
 # define POPminc(a)  0
 #endif
 
-#ifndef	RPOP
-# define RPOPminc(a) (a)
-#else
-# define RPOPminc(a)  0
-#endif
-
-#ifndef	APOP
-# define APOPminc(a) (a)
-#else
-# define APOPminc(a)  0
-#endif
-
-#ifndef	KPOP
-# define KPOPminc(a) (a)
-#else
-# define KPOPminc(a)  0
-#endif
-
 #ifndef CYRUS_SASL
 # define SASLminc(a) (a)
 #else
@@ -102,37 +78,29 @@ static struct swit switches[] = {
     { "pack file", POPminc (-4) },
 #define	NPACKSW                   10
     { "nopack", POPminc (-6) },
-#define	APOPSW                    11
-    { "apop", APOPminc (-4) },
-#define	NAPOPSW                   12
-    { "noapop", APOPminc (-6) },
-#define	RPOPSW                    13
-    { "rpop", RPOPminc (-4) },
-#define	NRPOPSW                   14
-    { "norpop", RPOPminc (-6) },
-#define	SILSW                     15
+#define PORTSW			  11
+    { "port name/number", POPminc (-4) },
+#define	SILSW                     12
     { "silent", 0 },
-#define	NSILSW                    16
+#define	NSILSW                    13
     { "nosilent", 0 },
-#define	TRNCSW                    17
+#define	TRNCSW                    14
     { "truncate", 0 },
-#define	NTRNCSW                   18
+#define	NTRNCSW                   15
     { "notruncate", 0 },
-#define	WIDTHSW                   19
+#define	WIDTHSW                   16
     { "width columns", 0 },
-#define VERSIONSW                 20
+#define VERSIONSW                 17
     { "version", 0 },
-#define	HELPSW                    21
+#define	HELPSW                    18
     { "help", 0 },
-#define SNOOPSW                   22
+#define SNOOPSW                   19
     { "snoop", -5 },
-#define KPOPSW                    23
-    { "kpop", KPOPminc (-4) },
-#define SASLSW                    24
+#define SASLSW                    20
     { "sasl", SASLminc(-4) },
-#define SASLMECHSW                25
+#define SASLMECHSW                21
     { "saslmech", SASLminc(-8) },
-#define PROXYSW                   26
+#define PROXYSW                   22
     { "proxy command", POPminc(-5) },
     { NULL, 0 }
 };
@@ -144,22 +112,29 @@ static struct swit switches[] = {
 #define INC_POP   1
 
 static int inc_type;
+static struct Maildir_entry {
+	char *filename;
+	time_t mtime;
+} *Maildir = NULL;
+static int num_maildir_entries = 0;
 static int snoop = 0;
 
 #ifdef POP
 extern char response[];
 
-static char *packfile = NULL;
 static int size;
 static long pos;
-static long start;
-static long stop;
 
 static int mbx_style = MMDF_FORMAT;
 static int pd = NOTOK;
-static FILE *pf = NULL;
 #endif /* POP */
 
+static long start;
+static long stop;
+
+static char *packfile = NULL;
+static FILE *pf = NULL;
+
 /* This is an attempt to simplify things by putting all the
  * privilege ops into macros.
  * *GROUPPRIVS() is related to handling the setgid MAIL property,
@@ -207,12 +182,6 @@ if (geteuid() != 0) DROPGROUPPRIVS()
 #define SAVEGROUPPRIVS()
 #endif /* not MAILGROUP */
 
-#ifdef POP
-#define DROPUSERPRIVS() setuid(getuid())
-#else
-#define DROPUSERPRIVS()
-#endif
-
 /* these variables have to be globals so that done() can correctly clean up the lockfile */
 static int locked = 0;
 static char *newmail;
@@ -223,34 +192,46 @@ static FILE *in;
  */
 char *map_name(char *);
 
-static void inc_done(int);
+static void inc_done(int) NORETURN;
 #ifdef POP
 static int pop_action(char *);
 static int pop_pack(char *);
 static int map_count(void);
 #endif
 
+int
+maildir_srt(const void *va, const void *vb)
+{
+    const struct Maildir_entry *a = va, *b = vb;
+    if (a->mtime > b->mtime)
+      return 1;
+    else if (a->mtime < b->mtime)
+      return -1;
+    else
+      return 0;
+}
 
 int
 main (int argc, char **argv)
 {
     int chgflag = 1, trnflag = 1;
     int noisy = 1, width = 0;
-    int rpop, i, hghnum, msgnum;
-    int kpop = 0, sasl = 0;
-    char *cp, *maildir, *folder = NULL;
+    int hghnum = 0, msgnum = 0;
+    int sasl = 0;
+    int incerr = 0; /* <0 if inc hits an error which means it should not truncate mailspool */
+    char *cp, *maildir = NULL, *folder = NULL;
     char *format = NULL, *form = NULL;
-    char *host = NULL, *user = NULL, *proxy = NULL;
+    char *host = NULL, *port = NULL, *user = NULL, *proxy = NULL;
     char *audfile = NULL, *from = NULL, *saslmech = NULL;
     char buf[BUFSIZ], **argp, *nfs, **arguments;
-    struct msgs *mp;
+    struct msgs *mp = NULL;
     struct stat st, s1;
     FILE *aud = NULL;
-    char	b[MAXPATHLEN + 1];
-    char	*maildir_copy;		/* copy of mail directory because the static gets overwritten */
+    char b[MAXPATHLEN + 1];
+    char *maildir_copy = NULL;	/* copy of mail directory because the static gets overwritten */
 
 #ifdef POP
-    int nmsgs, nbytes, p = 0;
+    int nmsgs, nbytes;
     char *pass = NULL;
     char *MAILHOST_env_variable;
 #endif
@@ -259,10 +240,6 @@ main (int argc, char **argv)
     FILE *mhe = NULL;
 #endif
 
-#ifdef HESIOD
-    struct hes_postoffice *po;
-#endif
-
     done=inc_done;
 
 /* absolutely the first thing we do is save our privileges,
@@ -293,11 +270,6 @@ main (int argc, char **argv)
      */
     if ((MAILHOST_env_variable = getenv("MAILHOST")) != NULL)
 	pophost = MAILHOST_env_variable;
-# ifdef HESIOD
-    else if ((po = hes_getmailhost(getusername())) != NULL &&
-	     strcmp(po->po_type, "POP") == 0)
-	pophost = po->po_host;
-# endif /* HESIOD */
     /*
      * If there is a valid "pophost" entry in mts.conf,
      * then use it as the default host.
@@ -309,8 +281,6 @@ main (int argc, char **argv)
 	snoop++;
 #endif /* POP */
 
-    rpop = 0;
-
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -399,6 +369,12 @@ main (int argc, char **argv)
 		if (!(host = *argp++) || *host == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
 		continue;
+
+	    case PORTSW:
+		if (!(host = *argp++) || *port == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		continue;
+
 	    case USERSW:
 		if (!(user = *argp++) || *user == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
@@ -419,24 +395,6 @@ main (int argc, char **argv)
 #endif /* POP */
 		continue;
 
-	    case APOPSW:
-		rpop = -1;
-		continue;
-	    case NAPOPSW:
-		rpop = 0;
-		continue;
-
-	    case RPOPSW:
-		rpop = 1;
-		continue;
-	    case NRPOPSW:
-		rpop = 0;
-		continue;
-
-	    case KPOPSW:
-		kpop = 1;
-		continue;
-
 	    case SNOOPSW:
 		snoop++;
 		continue;
@@ -471,8 +429,6 @@ main (int argc, char **argv)
 #ifdef POP
     if (host && !*host)
 	host = NULL;
-    if (from || !host || rpop <= 0)
-	DROPUSERPRIVS();
 #endif /* POP */
 
     /* guarantee dropping group priveleges; we might not have done so earlier */
@@ -498,10 +454,7 @@ main (int argc, char **argv)
     if (inc_type == INC_POP) {
 	if (user == NULL)
 	    user = getusername ();
-	if ( strcmp( POPSERVICE, "kpop" ) == 0 ) {
-	    kpop = 1;
-	}
-	if (kpop || sasl || ( rpop > 0))
+	if (sasl)
 	    pass = getusername ();
 	else
 	    ruserpass (host, &user, &pass);
@@ -509,16 +462,14 @@ main (int argc, char **argv)
 	/*
 	 * initialize POP connection
 	 */
-	if (pop_init (host, user, pass, proxy, snoop, kpop ? 1 : rpop, kpop,
-		      sasl, saslmech) == NOTOK)
+	if (pop_init (host, port, user, pass, proxy, snoop, sasl,
+		      saslmech) == NOTOK)
 	    adios (NULL, "%s", response);
 
 	/* Check if there are any messages */
 	if (pop_stat (&nmsgs, &nbytes) == NOTOK)
 	    adios (NULL, "%s", response);
 
-	if (rpop > 0)
-	    DROPUSERPRIVS();
 	if (nmsgs == 0) {
 	    pop_quit();
 	    adios (NULL, "no mail to incorporate");
@@ -543,9 +494,58 @@ main (int argc, char **argv)
 	}
 	if (stat (newmail, &s1) == NOTOK || s1.st_size == 0)
 	    adios (NULL, "no mail to incorporate");
+ 	if (s1.st_mode & S_IFDIR) {
+ 	    DIR *md;
+ 	    struct dirent *de;
+ 	    struct stat ms;
+ 	    int i;
+ 	    i = 0;
+ 	    cp = concat (newmail, "/new", NULL);
+ 	    if ((md = opendir(cp)) == NULL)
+ 		adios (NULL, "unable to open %s", cp);
+ 	    while ((de = readdir (md)) != NULL) {
+ 		if (de->d_name[0] == '.')
+ 		    continue;
+ 		if (i >= num_maildir_entries) {
+ 		    if ((Maildir = realloc(Maildir, sizeof(*Maildir) * (2*i+16))) == NULL)
+ 			adios(NULL, "not enough memory for %d messages", 2*i+16);
+ 		    num_maildir_entries = 2*i+16;
+ 		}
+ 		Maildir[i].filename = concat (cp, "/", de->d_name, NULL);
+ 		if (stat(Maildir[i].filename, &ms) != 0)
+ 	           adios (Maildir[i].filename, "couldn't get delivery time");
+ 		Maildir[i].mtime = ms.st_mtime;
+ 		i++;
+ 	    }
+ 	    free (cp);
+ 	    closedir (md);
+ 	    cp = concat (newmail, "/cur", NULL);
+ 	    if ((md = opendir(cp)) == NULL)
+ 		adios (NULL, "unable to open %s", cp);
+ 	    while ((de = readdir (md)) != NULL) {
+ 		if (de->d_name[0] == '.')
+ 		    continue;
+ 		if (i >= num_maildir_entries) {
+ 		    if ((Maildir = realloc(Maildir, sizeof(*Maildir) * (2*i+16))) == NULL)
+ 			adios(NULL, "not enough memory for %d messages", 2*i+16);
+ 		    num_maildir_entries = 2*i+16;
+ 		}
+ 		Maildir[i].filename = concat (cp, "/", de->d_name, NULL);
+ 		if (stat(Maildir[i].filename, &ms) != 0)
+ 	           adios (Maildir[i].filename, "couldn't get delivery time");
+ 		Maildir[i].mtime = ms.st_mtime;
+ 		i++;
+ 	    }
+ 	    free (cp);
+ 	    closedir (md);
+ 	    if (i == 0)
+ 	        adios (NULL, "no mail to incorporate");
+ 	    num_maildir_entries = i;
+ 	    qsort (Maildir, num_maildir_entries, sizeof(*Maildir), maildir_srt);
+ 	}
 
 	if ((cp = strdup(newmail)) == (char *)0)
-	    adios (maildir, "error allocating memory to copy newmail");
+	    adios (NULL, "error allocating memory to copy newmail");
 
 	newmail = cp;
     }
@@ -586,7 +586,7 @@ main (int argc, char **argv)
 go_to_it:
 #endif /* POP */
 
-    if (inc_type == INC_FILE) {
+    if (inc_type == INC_FILE && Maildir == NULL) {
 	if (access (newmail, W_OK) != NOTOK) {
 	    locked++;
 	    if (trnflag) {
@@ -613,6 +613,7 @@ go_to_it:
     DROPGROUPPRIVS();
 
     if (audfile) {
+	int i;
 	if ((i = stat (audfile, &st)) == NOTOK)
 	    advise (NULL, "Creating Receive-Audit: %s", audfile);
 	if ((aud = fopen (audfile, "a")) == NULL)
@@ -622,10 +623,9 @@ go_to_it:
 
 #ifdef POP
 	fprintf (aud, from ? "<<inc>> %s -ms %s\n"
-		 : host ? "<<inc>> %s -host %s -user %s%s\n"
+		 : host ? "<<inc>> %s -host %s -user %s\n"
 		 : "<<inc>> %s\n",
-		 dtimenow (0), from ? from : host, user,
-		 rpop < 0 ? " -apop" : rpop > 0 ? " -rpop" : "");
+		 dtimenow (0), from ? from : host, user);
 #else /* POP */
 	fprintf (aud, from ? "<<inc>> %s  -ms %s\n" : "<<inc>> %s\n",
 		 dtimenow (0), from);
@@ -634,6 +634,7 @@ go_to_it:
 
 #ifdef MHE
     if (context_find ("mhe")) {
+	int i;
 	cp = concat (maildir, "/++", NULL);
 	i = stat (cp, &st);
 	if ((mhe = fopen (cp, "a")) == NULL)
@@ -658,6 +659,7 @@ go_to_it:
      * Get the mail from a POP server
      */
     if (inc_type == INC_POP) {
+	int i;
 	if (packfile) {
 	    packfile = path (packfile, TFILE);
 	    if (stat (packfile, &st) == NOTOK) {
@@ -717,7 +719,7 @@ go_to_it:
 		    adios (cp, "write error on");
 		fseek (pf, 0L, SEEK_SET);
 	    }
-	    switch (p = scan (pf, msgnum, 0, nfs, width,
+	    switch (incerr = scan (pf, msgnum, 0, nfs, width,
 			      packfile ? 0 : msgnum == mp->hghmsg + 1 && chgflag,
 			      1, NULL, stop - start, noisy)) {
 	    case SCNEOF: 
@@ -790,10 +792,10 @@ go_to_it:
     /*
      * Get the mail from file (usually mail spool)
      */
-    if (inc_type == INC_FILE) {
+    if (inc_type == INC_FILE && Maildir == NULL) {
 	m_unknown (in);		/* the MAGIC invocation... */
 	hghnum = msgnum = mp->hghmsg;
-	for (i = 0;;) {
+	for (;;) {
 	    /*
 	     * Check if we need to allocate more space for message status.
 	     * If so, then add space for an additional 100 messages.
@@ -801,7 +803,7 @@ go_to_it:
 	    if (msgnum >= mp->hghoff
 		&& !(mp = folder_realloc (mp, mp->lowoff, mp->hghoff + 100))) {
 		advise (NULL, "unable to allocate folder storage");
-		i = NOTOK;
+		incerr = NOTOK;
 		break;
 	    }
 
@@ -819,7 +821,7 @@ go_to_it:
 	    newmsg = folder_addmsg(mp, tmpfilenam);
 #endif
 	    /* create scanline for new message */
-	    switch (i = scan (in, msgnum + 1, msgnum + 1, nfs, width,
+	    switch (incerr = scan (in, msgnum + 1, msgnum + 1, nfs, width,
 			      msgnum == hghnum && chgflag, 1, NULL, 0L, noisy)) {
 	    case SCNFAT:
 	    case SCNEOF: 
@@ -836,7 +838,7 @@ go_to_it:
 		break;
 
 	    default: 
-		advise (NULL, "BUG in %s, scan() botch (%d)", invo_name, i);
+		advise (NULL, "BUG in %s, scan() botch (%d)", invo_name, incerr);
 		break;
 
 	    case SCNMSG:
@@ -868,15 +870,118 @@ go_to_it:
 		mp->msgflags |= SEQMOD;
 		continue;
 	    }
+	    /* If we get here there was some sort of error from scan(),
+	     * so stop processing anything more from the spool.
+	     */
 	    break;
 	}
+    } else if (inc_type == INC_FILE) { /* Maildir inbox to process */
+	char *sp;
+	FILE *sf;
+	int i;
+
+	hghnum = msgnum = mp->hghmsg;
+	for (i = 0; i < num_maildir_entries; i++) {
+	    msgnum++;
+	    /*
+	     * Check if we need to allocate more space for message status.
+	     * If so, then add space for an additional 100 messages.
+	     */
+	    if (msgnum >= mp->hghoff
+		&& !(mp = folder_realloc (mp, mp->lowoff, mp->hghoff + 100))) {
+		advise (NULL, "unable to allocate folder storage");
+		incerr = NOTOK;
+		break;
+	    }
+
+	    sp = Maildir[i].filename;
+	    cp = getcpy (m_name (msgnum));
+	    pf = NULL;
+	    if (!trnflag || link(sp, cp) == -1) {
+		static char buf[65536];
+		size_t nrd;
+
+		if ((sf = fopen (sp, "r")) == NULL)
+	    	    adios (sp, "unable to read for copy");
+		if ((pf = fopen (cp, "w+")) == NULL)
+		    adios (cp, "unable to write for copy");
+		while ((nrd = fread(buf, 1, sizeof(buf), sf)) > 0)
+		    if (fwrite(buf, 1, nrd, pf) != nrd)
+			break;
+		if (ferror(sf) || fflush(pf) || ferror(pf)) {
+			int e = errno;
+			fclose(pf); fclose(sf); unlink(cp);
+			errno = e;
+			adios(cp, "copy error %s -> %s", sp, cp);
+		}
+		fclose (sf);
+		sf = NULL;
+	    } 
+	    if (pf == NULL && (pf = fopen (cp, "r")) == NULL)
+	        adios (cp, "not available");
+	    chmod (cp, m_gmprot ());
+
+	    fseek (pf, 0L, SEEK_SET);
+	    switch (incerr = scan (pf, msgnum, 0, nfs, width,
+			      msgnum == mp->hghmsg + 1 && chgflag,
+			      1, NULL, stop - start, noisy)) {
+	    case SCNEOF: 
+		printf ("%*d  empty\n", DMAXFOLDER, msgnum);
+		break;
+
+	    case SCNFAT:
+		trnflag = 0;
+		noisy++;
+		/* advise (cp, "unable to read"); already advised */
+		/* fall thru */
+
+	    case SCNERR:
+	    case SCNNUM: 
+		break;
+
+	    case SCNMSG: 
+	    case SCNENC:
+	    default: 
+		/*
+		 *  Run the external program hook on the message.
+		 */
+
+		(void)snprintf(b, sizeof (b), "%s/%d", maildir_copy, msgnum + 1);
+		(void)ext_hook("add-hook", b, (char *)0);
+
+		if (aud)
+		    fputs (scanl, aud);
+# ifdef MHE
+		if (mhe)
+		    fputs (scanl, mhe);
+# endif /* MHE */
+		if (noisy)
+		    fflush (stdout);
+		if (!packfile) {
+		    clear_msg_flags (mp, msgnum);
+		    set_exists (mp, msgnum);
+		    set_unseen (mp, msgnum);
+		    mp->msgflags |= SEQMOD;
+		}
+		break;
+	    }
+	    if (ferror(pf) || fclose (pf)) {
+		int e = errno;
+		unlink (cp);
+		errno = e;
+		adios (cp, "write error on");
+	    }
+	    pf = NULL;
+	    free (cp);
+
+	    if (trnflag && unlink (sp) == NOTOK)
+		adios (sp, "couldn't unlink");
+	    free (sp); /* Free Maildir[i]->filename */
+	}
+	free (Maildir); /* From now on Maildir is just a flag - don't dref! */
     }
 
-#ifdef POP
-    if (p < 0) {		/* error */
-#else
-    if (i < 0) {		/* error */
-#endif
+    if (incerr < 0) {		/* error */
 	if (locked) {
             GETGROUPPRIVS();      /* Be sure we can unlock mail file */
             (void) lkfclose (in, newmail); in = NULL;
@@ -906,13 +1011,14 @@ go_to_it:
     /*
      * truncate file we are incorporating from
      */
-    if (inc_type == INC_FILE) {
+    if (inc_type == INC_FILE && Maildir == NULL) {
 	if (trnflag) {
 	    if (stat (newmail, &st) != NOTOK && s1.st_mtime != st.st_mtime)
 		advise (NULL, "new messages have arrived!\007");
 	    else {
-		if ((i = creat (newmail, 0600)) != NOTOK)
-		    close (i);
+		int newfd;
+		if ((newfd = creat (newmail, 0600)) != NOTOK)
+		    close (newfd);
 		else
 		    admonish (newmail, "error zero'ing");
 		unlink(map_name(newmail));
@@ -939,7 +1045,7 @@ go_to_it:
     /*
      * unlock the mail spool
      */
-    if (inc_type == INC_FILE) {
+    if (inc_type == INC_FILE && Maildir == NULL) {
 	if (locked) {
            GETGROUPPRIVS();        /* Be sure we can unlock mail file */
            (void) lkfclose (in, newmail); in = NULL;
