@@ -2,8 +2,6 @@
 /*
  * mhbuild.c -- expand/translate MIME composition files
  *
- * $Id: mhbuild.c,v 1.12 2007/11/04 11:54:34 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -74,7 +72,6 @@ static struct swit switches[] = {
 
 
 /* mhbuildsbr.c */
-extern int checksw;
 extern char *tmp;	/* directory to place temp files */
 
 /* mhcachesbr.c */
@@ -100,11 +97,12 @@ static int unlink_infile  = 0;
 static char outfile[BUFSIZ];
 static int unlink_outfile = 0;
 
-static void unlink_done (int);
+static void unlink_done (int) NORETURN;
 
 /* mhbuildsbr.c */
 CT build_mime (char *);
 int output_message (CT, char *);
+int output_message_fp (CT, FILE *, char*);
 
 /* mhlistsbr.c */
 int list_all_messages (CT *, int, int, int, int);
@@ -125,7 +123,8 @@ main (int argc, char **argv)
     char buffer[BUFSIZ], *compfile = NULL;
     char **argp, **arguments;
     CT ct, cts[2];
-    FILE *fp;
+    FILE *fp = NULL;
+    FILE *fp_out = NULL;
 
     done=unlink_done;
 
@@ -306,11 +305,8 @@ main (int argc, char **argv)
      * Process the composition file from standard input.
      */
     if (compfile[0] == '-' && compfile[1] == '\0') {
-
 	/* copy standard input to temporary file */
-	strncpy (infile, m_scratch ("", invo_name), sizeof(infile));
-	if ((fp = fopen (infile, "w")) == NULL)
-	    adios (infile, "unable to open");
+	strncpy (infile, m_mktemp(invo_name, NULL, &fp), sizeof(infile));
 	while (fgets (buffer, BUFSIZ, stdin))
 	    fputs (buffer, fp);
 	fclose (fp);
@@ -322,11 +318,12 @@ main (int argc, char **argv)
 	cts[1] = NULL;
 
 	/* output MIME message to this temporary file */
-	strncpy (outfile, m_scratch ("", invo_name), sizeof(outfile));
+	strncpy (outfile, m_mktemp(invo_name, NULL, &fp_out), sizeof(outfile));
 	unlink_outfile = 1;
 
 	/* output the message */
-	output_message (ct, outfile);
+	output_message_fp (ct, fp_out, outfile);
+        fclose(fp_out);
 
 	/* output the temp file to standard output */
 	if ((fp = fopen (outfile, "r")) == NULL)
@@ -355,11 +352,13 @@ main (int argc, char **argv)
     cts[1] = NULL;
 
     /* output MIME message to this temporary file */
-    strncpy (outfile, m_scratch (compfile, invo_name), sizeof(outfile));
+    strncpy(outfile, m_mktemp2(compfile, invo_name, NULL, &fp_out),
+            sizeof(outfile));
     unlink_outfile = 1;
 
     /* output the message */
-    output_message (ct, outfile);
+    output_message_fp (ct, fp_out, outfile);
+    fclose(fp_out);
 
     /*
      * List the message info
@@ -369,12 +368,13 @@ main (int argc, char **argv)
 
     /* Rename composition draft */
     snprintf (buffer, sizeof(buffer), "%s.orig", m_backup (compfile));
-    if (rename (compfile, buffer) == NOTOK)
-	adios (compfile, "unable to rename %s to", buffer);
+    if (rename (compfile, buffer) == NOTOK) {
+	adios (compfile, "unable to rename comp draft %s to", buffer);
+    }
 
     /* Rename output file to take its place */
     if (rename (outfile, compfile) == NOTOK) {
-	advise (outfile, "unable to rename %s to", compfile);
+	advise (outfile, "unable to rename output %s to", compfile);
 	rename (buffer, compfile);
 	done (1);
     }
