@@ -2,8 +2,6 @@
 /*
  * mhn.c -- display, list, cache, or store the contents of MIME messages
  *
- * $Id: mhn.c,v 1.15 2007/11/04 11:54:34 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -126,7 +124,6 @@ static struct swit switches[] = {
 
 
 /* mhparse.c */
-extern int checksw;
 extern char *tmp;	/* directory to place temp files */
 
 /* mhcachesbr.c */
@@ -198,7 +195,7 @@ void cache_all_messages (CT *);
 /* mhfree.c */
 void free_content (CT);
 extern CT *cts;
-void freects_done (int);
+void freects_done (int) NORETURN;
 
 /*
  * static prototypes
