@@ -3,8 +3,6 @@
  * mhoutsbr.c -- routines to output MIME messages
  *            -- given a Content structure
  *
- * $Id: mhoutsbr.c,v 1.8 2006/03/08 12:14:16 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -70,6 +68,7 @@ static char nib2b64[0x40+1] =
  * prototypes
  */
 int output_message (CT, char *);
+int output_message_fp (CT, FILE *, char *);
 int writeBase64aux (FILE *, FILE *);
 
 /*
@@ -89,26 +88,32 @@ static int writeBase64 (CT, FILE *);
  * transfer encoding is added.
  */
 
+int
+output_message_fp (CT ct, FILE *fp, char *file)
+{
+    if (output_content (ct, fp) == NOTOK)
+	return NOTOK;
+
+    if (fflush (fp)) {
+	advise ((file?file:"<FILE*>"), "error writing to");
+	return NOTOK;
+    }
+    return OK;
+}
+
 int
 output_message (CT ct, char *file)
 {
     FILE *fp;
+    int status;
 
     if ((fp = fopen (file, "w")) == NULL) {
 	advise (file, "unable to open for writing");
 	return NOTOK;
     }
-
-    if (output_content (ct, fp) == NOTOK)
-	return NOTOK;
-
-    if (fflush (fp)) {
-	advise (file, "error writing to");
-	return NOTOK;
-    }
-    fclose (fp);
-
-    return OK;
+    status = output_message_fp(ct, fp, file);
+    fclose(fp);
+    return status;
 }
 
 
