@@ -2,8 +2,6 @@
 /*
  * picksbr.c -- routines to help pick along...
  *
- * $Id: picksbr.c,v 1.12 2007/04/29 22:57:36 pm215 Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -538,7 +536,8 @@ PRaction (struct nexus *n, int level)
 		dasctime (&n->n_tws, TW_NULL));
 	return;
     }
-    fprintf (stderr, "UNKNOWN(0x%x)\n", (unsigned int) (*n->n_action));
+    fprintf (stderr, "UNKNOWN(0x%x)\n",
+	     (unsigned int)(unsigned long) (*n->n_action));
 }
 
 
