@@ -2,8 +2,6 @@
 /*
  * prompter.c -- simple prompting editor front-end
  *
- * $Id: prompter.c,v 1.6 2007/11/04 11:54:35 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -108,6 +106,7 @@ main (int argc, char **argv)
     char buffer[BUFSIZ], tmpfil[BUFSIZ];
     char **arguments, **argp;
     FILE *in, *out;
+    char *tfile = NULL;
 
 #ifdef LOCALE
     setlocale(LC_ALL, "");
@@ -185,10 +184,10 @@ main (int argc, char **argv)
     if ((in = fopen (drft, "r")) == NULL)
 	adios (drft, "unable to open");
 
-    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
-    if ((out = fopen (tmpfil, "w")) == NULL)
-	adios (tmpfil, "unable to create");
+    tfile = m_mktemp2(NULL, invo_name, NULL, &out);
+    if (tfile == NULL) adios("prompter", "unable to create temporary file");
     chmod (tmpfil, 0600);
+    strncpy (tmpfil, tfile, sizeof(tmpfil));
 
     /*
      * Are we changing the kill or erase character?
