@@ -2,8 +2,6 @@
 /*
  * rcvstore.c -- asynchronously add mail to a folder
  *
- * $Id: rcvstore.c,v 1.13 2007/11/04 11:54:36 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -48,7 +46,7 @@ static struct swit switches[] = {
  */
 static char *tmpfilenam = NULL;
 
-static void unlink_done(int);
+static void unlink_done(int) NORETURN;
 
 int
 main (int argc, char **argv)
@@ -173,9 +171,10 @@ main (int argc, char **argv)
     SIGNAL (SIGTERM, SIG_IGN);
 
     /* create a temporary file */
-    tmpfilenam = m_scratch ("", invo_name);
-    if ((fd = creat (tmpfilenam, m_gmprot ())) == NOTOK)
-	adios (tmpfilenam, "unable to create");
+    tmpfilenam = m_mktemp (invo_name, &fd, NULL);
+    if (tmpfilenam == NULL) {
+	adios ("rcvstore", "unable to create temporary file");
+    }
     chmod (tmpfilenam, m_gmprot());
 
     /* copy the message from stdin into temp file */
