@@ -2,8 +2,6 @@
 /*
  * send.c -- send a composed message
  *
- * $Id: send.c,v 1.9 2007/11/04 11:54:36 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -21,6 +19,12 @@
 # define SASLminc(a)  0
 #endif /* CYRUS_SASL */
 
+#ifndef TLS_SUPPORT
+# define TLSminc(a)  (a)
+#else /* TLS_SUPPORT */
+# define TLSminc(a)   0
+#endif /* TLS_SUPPORT */
+
 static struct swit switches[] = {
 #define	ALIASW                 0
     { "alias aliasfile", 0 },
@@ -93,19 +97,23 @@ static struct swit switches[] = {
 #define	CLIESW                34
     { "client host", -6 },
 #define	SERVSW                35
-    { "server host", -6 },
+    { "server host", 6 },
 #define	SNOOPSW               36
-    { "snoop", -5 },
+    { "snoop", 5 },
 #define SASLSW                37
-    { "sasl", SASLminc(-4) },
+    { "sasl", SASLminc(4) },
 #define SASLMECHSW            38
-    { "saslmech", SASLminc(-5) },
+    { "saslmech mechanism", SASLminc(-5) },
 #define USERSW                39
-    { "user", SASLminc(-4) },
+    { "user username", SASLminc(-4) },
 #define ATTACHSW              40
     { "attach", 6 },
 #define ATTACHFORMATSW        41
     { "attachformat", 7 },
+#define PORTSW		      42
+    { "port server-port-name/number" , 4 },
+#define TLSSW		      43
+    { "tls", TLSminc(-3) },
     { NULL, 0 }
 };
 
@@ -263,6 +271,7 @@ main (int argc, char **argv)
 		case SOMLSW: 
 		case SNOOPSW: 
 		case SASLSW:
+		case TLSSW:
 		    vec[vecp++] = --cp;
 		    continue;
 
@@ -273,6 +282,7 @@ main (int argc, char **argv)
 		case SERVSW: 
 		case SASLMECHSW:
 		case USERSW:
+		case PORTSW:
 		    vec[vecp++] = --cp;
 		    if (!(cp = *argp++) || *cp == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
@@ -420,7 +430,7 @@ go_to_it:
 	    && (distsw = atoi (cp))
 	    && altmsg) {
 	vec[vecp++] = "-dist";
-	distfile = getcpy (m_scratch (altmsg, invo_name));
+	distfile = getcpy (m_mktemp2 (altmsg, invo_name, NULL, NULL));
 	if (link (altmsg, distfile) == NOTOK) {
 	    if (errno != EXDEV
 #ifdef EISREMOTE
@@ -429,7 +439,7 @@ go_to_it:
 		)
 		adios (distfile, "unable to link %s to", altmsg);
 	    free (distfile);
-	    distfile = getcpy (m_tmpfil (invo_name));
+	    distfile = getcpy (m_mktemp2(NULL, invo_name, NULL, NULL));
 	    {
 		int in, out;
 		struct stat st;
